@@ -1,7 +1,10 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001  John Elliott <jce@seasip.demon.co.uk>            *
+ *    Copyright (C) 2001-2017 John Elliott <seasip.webmaster@gmail.com>    *
+ *                                                                         *
+ *    Modifications to add dsk_dirty()                                     *
+ *    (c) 2005 Philip Kendall <pak21-spectrum@srcf.ucam.org>               *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -55,7 +58,7 @@
 extern "C" {
 #endif
 
-#define LIBDSK_VERSION "1.1.1"
+#define LIBDSK_VERSION "1.5.4"
 
 /************************* TYPES ********************************/
 
@@ -93,10 +96,16 @@ typedef const char *  dsk_cchar_t;	/* Const char * */
 #define DSK_ERR_RPC	 (-25)  /* Error encoding / decoding RPC */
 #define DSK_ERR_BADOPT	 (-26)  /* Requested optional feature not present */
 #define DSK_ERR_BADVAL	 (-27)  /* Bad value for requested option */
+#define DSK_ERR_ABORT    (-28)  /* User abort requested */
+#define DSK_ERR_TIMEOUT  (-29)  /* Communications timed out */
+#define DSK_ERR_UNKRPC   (-30)  /* RPC server does not recognise function */
+#define DSK_ERR_BADMEDIA (-31)	/* Unsuitable media for drive */
+#define DSK_ERR_CORRUPT  (-32)	/* Disk image file is corrupt */
 #define DSK_ERR_UNKNOWN  (-99)	/* Unknown error */
 
 /* Is this error a transient error, that may be cleared by a retry? */
-#define DSK_TRANSIENT_ERROR(e) ((e) < DSK_ERR_COMPRESS && (e) > DSK_ERR_NOTRDY)
+/* 1.1.3: Get this the right way round; they're negative numbers! */
+#define DSK_TRANSIENT_ERROR(e) ((e) > DSK_ERR_COMPRESS && (e) <= DSK_ERR_NOTRDY)
 
 /* Disc sidedness (logical/physical mapping). Use SIDES_ALT for single-sided floppies. */
 typedef enum 
@@ -104,8 +113,11 @@ typedef enum
 	SIDES_ALT, 	/* Track n is cylinder (n/heads) head (n%heads) */
 	SIDES_OUTBACK, 	/* Tracks go (head 0) 0,1,2,3,...37,38,39, then
 			             (head 1) 39,38,37,...,2,1,0 */
-	SIDES_OUTOUT	/* Tracks go (head 0) 0,1,2,3,...37,38,39, then
+	SIDES_OUTOUT,	/* Tracks go (head 0) 0,1,2,3,...37,38,39, then
 			             (head 1) 0,1,2,3,...37,38,39 */
+	SIDES_EXTSURFACE/* As SIDES_ALT, but sectors on head 1 identify
+			 * as head 0, with numbers in sequence 
+			 * eg: Head 0 has sectors 1-9, head 1 has 10-18 */
 } dsk_sides_t;
 
 typedef enum
@@ -139,9 +151,28 @@ typedef enum
 	FMT_BBC100,	/* 10 sectors, 40 tracks, 1 side, FM */
 	FMT_BBC200,	/* 10 sectors, 80 tracks, 1 side, FM */
 	FMT_MBEE400,	/* 10 sectors, 80 tracks, 1 side */
+	FMT_MGT800,     /* 10 sectors, 80 tracks, 2 sides out and out */
+	FMT_TRDOS640,	/* 16 sectors,  256 bytes/sector, 2 sides */
+	FMT_AMPRO200,	/* 10 sectors, 512 bytes/sector, 1 side */
+	FMT_AMPRO400D,	/* 10 sectors, 512 bytes/sector, 2 sides */
+	FMT_AMPRO400S,	/* 5 sectors, 1024 bytes/sector, 1 side */
+	FMT_AMPRO800,	/* 5 sectors, 1024 bytes/sector, 2 sides */
+	FMT_1200K,	/* 15 sectors, 2 sides  */
 	FMT_UNKNOWN = -1
 } dsk_format_t;
 
+typedef enum
+{
+	/* Low byte of dg_fm: Recording mode */
+	RECMODE_MASK     = 0x00FF,
+	RECMODE_MFM      = 0x0000,	
+	RECMODE_FM       = 0x0001,
+	/* High byte of dg_fm: Other data recording flags */
+	RECMODE_FLAGMASK = 0xFF00,
+	RECMODE_COMPLEMENT = 0x0100	
+	
+} dsk_recmode_t;
+
 /* DSK_GEOMETRY holds information used to convert physical to/from logical
  * sectors and to access the disc */
 typedef struct
@@ -155,8 +186,8 @@ typedef struct
 	dsk_rate_t	dg_datarate;	/* Data rate */
 	dsk_gap_t	dg_rwgap;	/* Read/write gap length */
 	dsk_gap_t	dg_fmtgap;	/* Format gap length */
-	int		dg_fm;		/* FM mode? The only thing I know that uses FM mode
-					 * is the BBC Micro DFS format. */
+	int		dg_fm;		/* Really a dsk_recmode_t, kept as
+					 * int for backward compatibility  */
 	int		dg_nomulti;	/* Disable multitrack? */
 	int		dg_noskip;	/* Set to 0 to skip deleted data */
 } DSK_GEOMETRY;
@@ -223,7 +254,7 @@ LDPUBLIC32 void LDPUBLIC16 dsk_reportfunc_get(DSK_REPORTFUNC *report,
  * operation such as decompression */
 LDPUBLIC32 void LDPUBLIC16 dsk_report(const char *s);
 /* Remove a work-in-progress message, if appropriate */
-LDPUBLIC32 void LDPUBLIC16 dsk_report_end();
+LDPUBLIC32 void LDPUBLIC16 dsk_report_end(void);
 
 /*****************************************************************************/
 
@@ -262,6 +293,10 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_creat(DSK_PDRIVER *self, const char *filena
 
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_close(DSK_PDRIVER *self);
 
+/* Returns whether the disk has been modified since it was opened */
+
+LDPUBLIC32 int              LDPUBLIC16 dsk_dirty(DSK_PDRIVER self);
+
 /* Get drive status (Ready, Read-Only etc.). The actual status is 
  * based on the FDC's ST3 register. The following bits should be available: */
 
@@ -364,6 +399,13 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_dosgeom(DSK_GEOMETRY *self, const unsigned c
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_pcwgeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_cpm86geom(DSK_GEOMETRY *self, const unsigned char *bootsect);
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_aprigeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_opusgeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
+
+/* Tries all the above, in approximate order of detectability. This is 
+ * intended to assist in parsing flat file disc images that contain no
+ * metadata, so the first 512 bytes are all you have to determine the
+ * format. */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dg_bootsecgeom(DSK_GEOMETRY *self, const unsigned char *bootsect);
 
 /* Read a random sector header from current track */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_psecid(DSK_PDRIVER self, const DSK_GEOMETRY *geom,
@@ -372,6 +414,23 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_psecid(DSK_PDRIVER self, const DSK_GEOMETRY
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_lsecid(DSK_PDRIVER self, const DSK_GEOMETRY *geom,
 				dsk_ltrack_t track, DSK_FORMAT *result);
 
+/* Read all sector headers from current track in the order they appear. 
+ * Not implemented yet; this is for future expansion. */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_ptrackids(DSK_PDRIVER self, 
+				const DSK_GEOMETRY *geom,
+				dsk_pcyl_t cylinder, dsk_phead_t head,
+				dsk_psect_t *count, DSK_FORMAT **results);
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_ltrackids(DSK_PDRIVER self, 
+				const DSK_GEOMETRY *geom,
+				dsk_ltrack_t track, 
+				dsk_psect_t *count, DSK_FORMAT **result);
+
+/* Read a track as it appears on the disk, including sector headers.
+ * Not implemented yet; this is for future expansion. */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_rtread(DSK_PDRIVER self, 
+		const DSK_GEOMETRY *geom, void *buf, 
+	        dsk_pcyl_t cylinder,     dsk_phead_t head, int reserved);
+
 /* Seek to a cylinder */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_lseek(DSK_PDRIVER self, const DSK_GEOMETRY *geom, 
 				dsk_ltrack_t track);
@@ -382,6 +441,10 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_pseek(DSK_PDRIVER self, const DSK_GEOMETRY
  * Else sets (*drvname) to null. */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_type_enum(int idx, char **drvname);
 
+/* If "index" is in range, returns the human-readable description of the 
+ * n'th driver name in (*drvdesc).  Else sets (*drvdesc) to null. */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_typedesc_enum(int idx, char **drvdesc);
+
 /* If "index" is in range, returns the n'th compressor name in (*compname).
  * Else sets (*drvname) to null. */
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_comp_enum(int idx, char **compname);
@@ -435,18 +498,31 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_map_itod(unsigned int n, DSK_PDRIVER *ptr);
  *  * all the memory used by the mapping */
 LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_map_delete(unsigned int index);
 
-
+/* Copy one entire disk image to another. Currently this can only be done if 
+ * both of them are based internally on LDBS; DSK_ERR_NOTIMPL will be returned 
+ * otherwise. 
+ * 
+ * geom can be null. If it is not null, it will be used when converting 
+ * from file formats that don't specify their own geometry (such as raw)
+ */
+LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_copy(DSK_GEOMETRY *geom, 
+				DSK_PDRIVER source, DSK_PDRIVER dest);
 
 /* Define this to print on the console a trace of all mallocs */
 #undef TRACE_MALLOCS 
 #ifdef TRACE_MALLOCS
 void *dsk_malloc(size_t size);
+void *dsk_remalloc(void *ptr, size_t size);
 void  dsk_free(void *ptr);
 #else
 #define dsk_malloc malloc
+#define dsk_realloc realloc
 #define dsk_free free
 #endif
 
+/* Helper method: Copy string 's' to a dsk_malloced buffer */
+LDPUBLIC32 char * LDPUBLIC16 dsk_malloc_string(const char *s);
+
 #ifdef __cplusplus
 }
 #endif
